#' Predict Method for local linear regressions
#' @name predict
#' @param object An object of class ``llreg"
#' @param COVARIATES A numeric matrix of n rows and p columns containing the predictors.
#' @param ... Ignored.
#' @return A numeric vector. 

predict.llreg = function(object,COVARIATES,...){
  
  Responses = object$Responses
  nind = length(Responses)
  
  unexpectedarguments = names(list(...))
  
  if(length(unexpectedarguments)==1){warning(paste("The argument",unexpectedarguments,"has been ignored"))}
  if(length(unexpectedarguments)>1){warning(paste("The arguments",paste(unexpectedarguments,", "),"have been ignored"))}
  
  COVARIATES = as.matrix(COVARIATES)
  
  if(object$standard){
    
    Meancovariates   = apply(object$COVARIATES, 2, mean)
    CCOVARIATES      = t(t(object$COVARIATES) - Meancovariates)
    CCOVARIATES_Test = t(t(COVARIATES)   - Meancovariates)
    
    Stdev         = sqrt(apply(CCOVARIATES^2, 2, mean))
    STDCOVAR      = t(t(CCOVARIATES) / Stdev)
    STDCOVAR_Test = t(t(CCOVARIATES_Test) / Stdev)
    
  }else{
    
    STDCOVAR_Test = COVARIATES
    STDCOVAR      = object$COVARIATES
    
  }
  
  bwknn.opt = ceiling(object$bwknn.ratio * nrow(STDCOVAR))
  
  D2 = 0
  for(jj in 1:ncol(STDCOVAR)) D2 = D2 + outer(STDCOVAR_Test[, jj], STDCOVAR[, jj], "-")^2
  PROX.NEW = sqrt(D2)
  Bw.new = apply(PROX.NEW, 1, sort)[bwknn.opt, ]
  Is.zero = (Bw.new == 0)
  if(sum(Is.zero) > 0){
    bw.min = min(Bw.new[!Is.zero])
    Bw.new[Is.zero] = 0.5 * bw.min
  }
  DIST = PROX.NEW / Bw.new
  KERNEL = 1 - DIST^2
  KERNEL[KERNEL < 0] = 0
  KERNEL[KERNEL > 1] = 0
  Denom = apply(KERNEL, 1, sum)
  Is.zero = (Denom == 0)
  if(sum(Is.zero) > 0){
    KERNEL[Is.zero, ] = 1
    Denom[Is.zero, ] = nind
  }
  Ynum = as.vector(KERNEL %*% Responses) 
  Ybar = Ynum / Denom
  T11 = crossprod(t(KERNEL) * Responses[,1], STDCOVAR)
  NUM = KERNEL %*% STDCOVAR
  XBAR = NUM / Denom
  T12 = XBAR * Ynum
  T1 = T11 - T12
  Predictions = 0
  for(ii in 1:nrow(STDCOVAR_Test)){
    CSTDCOVAR1 = t(t(STDCOVAR) - XBAR[ii,])
    T2 = crossprod(CSTDCOVAR1, (CSTDCOVAR1 * KERNEL[ii, ]))
    if(rcond(T2) < 1e-10){
      Predictions[ii] = Ybar[ii]
    }else{
      Bb = solve(T2, T1[ii, ])
      CSTDCOVAR2 = t(t(STDCOVAR_Test) - XBAR[ii, ])
      Predictions[ii] = Ybar[ii] + crossprod(Bb, CSTDCOVAR2[ii, ])
    } 
  }
  
  return(Predictions)  
  
}