#' Robustify novas output by exhaustively searching over all features
#' @name robustify
#' @param X An object of class novas.
#' 
#' @return An object of class novas.

robustify = function(X){
  
  j<-kk<-NULL
  
  COVARIATES = X$COVARIATES
  Responses  = X$llreg$Responses
  p = ncol(COVARIATES)
  n = nrow(COVARIATES)
  if(X$llreg$standard){
    Meancovariates = apply(COVARIATES,2,mean)
    CCOVARIATES = t(t(COVARIATES)-Meancovariates)
    Stdev = sqrt(apply(CCOVARIATES^2,2,mean))
    STDCOVAR = t(t(CCOVARIATES)/Stdev)
  }else{
    STDCOVAR = COVARIATES
  }
  args = X$args
  
  if(p>1){
    active.subset.size = p
    Grid = 1:active.subset.size
    Ploocvmin = numeric(0)
    ploocv.ini = sum((Responses-mean(Responses))^2)/n
    if(args$screening) cat("Forward addition in progress","\n")
    #############################################################################
    # STEP 1: ranking variables according to: loocv criterion when ploocvpar=0
    #                                         penalized loocv criterion otherwise
    #############################################################################
    Loocv = foreach(j = Grid, .combine='c', .export="llreg") %do% {
      loocv = llreg2(Responses, matrix(STDCOVAR[,j],ncol=1), X$llreg$bwknn.ratio)[[2]]
      return(loocv)
    }
    Ploocv = Loocv * (1 + args$ploocvpar / log(n))
    Ordered = order(Ploocv)
    posminploocv = Ordered[1]
    Ploocvmin[1] = Ploocv[posminploocv]
    Old.forward.selection = Grid[posminploocv]
    UpdatedGrid = Grid[-posminploocv]
    print(Ploocvmin[1])
    print(ploocv.ini)
    if(args$screening){
      if(Ploocvmin[1] > ploocv.ini) {
        stop("No design point selected: stepwise algorithm over")
      } else {
        cat(paste("Design point",Old.forward.selection,"selected"),"\n")
      }	
    }
    ####################################################
    # STEP >=2: selecting new variables?  
    ####################################################
    iter = 1
    epsilon = 2*args$threshold
    maxiter = min(args$maxsteps,p)
    while((epsilon > args$threshold) && (iter < maxiter)){
      iter = iter + 1
      Loocv = foreach(kk = UpdatedGrid, .combine='c', .export="llreg") %do% {
        loocv = llreg2(Responses, STDCOVAR[,c(Old.forward.selection, kk)], X$llreg$bwknn.ratio)[[2]]
        return(loocv)
      }
      model.size = length(Old.forward.selection) + 1
      Ploocv = Loocv * (1 + model.size * args$ploocvpar / log(n))
      Ordered = order(Ploocv)
      posminploocv = Ordered[1]
      Ploocvmin[iter] = Ploocv[posminploocv]
      New.forward.selection = c(Old.forward.selection, UpdatedGrid[posminploocv])
      epsilon = (Ploocvmin[iter-1]-Ploocvmin[iter])/Ploocvmin[iter-1]
      if(epsilon <= args$threshold){
        minimum.reached.for.ploocv = "yes"
      }else{
        if(args$screening) cat("Design point",UpdatedGrid[posminploocv],"selected","\n")
        minimum.reached.for.ploocv = "no"
        Old.forward.selection = New.forward.selection
        UpdatedGrid = UpdatedGrid[-posminploocv]
      }
    }
    
    Selectedmodel = Old.forward.selection
    
  }else{
    
    Selectedmodel = 1
    
  }
  
  model.size    = length(Selectedmodel)
  res.llreg     = llreg2(Responses, matrix(STDCOVAR[, Selectedmodel],ncol=model.size), X$llreg$bwknn.ratio)
  names(res.llreg) = c("Interpolated.values" , "loocv", "bwknn.ratio")
  res.llreg[["Responses"]]  = Responses
  res.llreg[["COVARIATES"]] = matrix(COVARIATES[, Selectedmodel],ncol=model.size)
  res.llreg[["standard"]]   = X$llreg$standard
  
  ploocvmin = res.llreg$loocv*(1 + model.size * args$ploocvpar / log(n))
  
  if(args$screening) cat("Final selected model =", Selectedmodel, " and ploocv =", ploocvmin,"\n")
  
  object = list(llreg = structure(res.llreg,class="llreg"), model = Selectedmodel, COVARIATES = COVARIATES, args = args)		
  
  class(object) = "novas"
  
  return(structure(object,class="novas"))
  
}