% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/novas.R
\name{novas}
\alias{novas}
\title{Most-predictive covariate-selection for nonparametric regression of a scalar response against multivariate predictors.}
\usage{
novas(
  Responses,
  COVARIATES,
  Bwknn.ratio = c(0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7),
  maxsteps = 10,
  exponent = 0.5,
  threshold = 0.05,
  standard = T,
  ploocvpar = 0,
  screening = T
)
}
\arguments{
\item{Responses}{A numeric vector of length n containing the responses Y1,...,Yn.}

\item{COVARIATES}{A numeric matrix of n rows and p columns containing the predictors.}

\item{Bwknn.ratio}{A vector of numerics in the [0,1] range containing the $k$-nearest-neighbour (KNN) ratio bandwidths to be investigated. 
It defaults to \code{Bwknn.ratio = c(0.1,0.2,0.3,0.4,0.5,0.6,0.7)}.}

\item{maxsteps}{A positive integer giving a bound on the cardinality of the sets of covariates to be investigated. It defaults to 10.}

\item{exponent}{A numeric in the [0,1] range governing the efficiency vs computational complexity tradeoff. At each iteration $p^exponent$ models will be maintained; lower values of \code{exponent} therefore make Novas faster. The default value, 0.5, corresponds to the choice of parameter made in Ferraty and Hall (2015).}

\item{threshold}{A numeric in the [0,1] range giving the stopping criterion. NOVAS will stop when the improvement 
gained from adding an additional covariate is smaller than (100 * threshold) \%. The default is 0.05.}

\item{standard}{A boolean. If \code{standard = TRUE} (the default), the matrix \code{COVARIATES} is standardised.}

\item{ploocvpar}{A non-negative numeric indicating the penalty to be added to the leave one out (LOO) CV. When set to zero (the default) un-penalised LOO-CV is used.}

\item{screening}{A boolean. If \code{screening = TRUE} the selected set of covariates along with the evaluation criterion are printed after each iteration.}
}
\description{
NOnparametric VAriable Selection (NOVAS) by Ferraty and Hall (2015) selects the most-predictive covariates by means of a stepwise forward strategy: At each step the most-predictive covariate
}
\examples{
### Simulated data example

library(Novas)
set.seed(1)

n = 200
p = 3
COVARIATES = matrix(rnorm(n*p),nrow = n,ncol = p)
Responses  = matrix(COVARIATES[,2]^2,ncol=1)+rnorm(n)
Out = novas(Responses, COVARIATES,ploocvpar = 0.05)
Robust_Out = robustify(Out)
summary(Robust_Out)

### Real data example

library(Novas)
data(pork)

pork_response       = pork$pork_response
pork_spec_curves    = pork$pork_spec_curves

pork_results = novas(pork_response, pork_spec_curves,threshold = 0.2)

# visualise the most predictive wavelengths
par(mfrow = c(1, 1), mar = c(5, 5, 1, 1), cex.axis = 1.5, cex.lab = 1.5)
spec_curves.ts = ts(t(pork_spec_curves), start = 850, frequency = 0.5)
ts.plot(spec_curves.ts, gpars = list(col = gray(0.5), xlab = "wavelengths", ylab = "absorbance"))
x = seq(850, by = 2, length = 100)
abline(v = x[pork_results$model], lty = 1, lwd = 2)


}
\references{
\insertRef{ferraty2015algorithm}{Novas}
}
