// -*- mode: C++; c-indent-level: 4; c-basic-offset: 4; indent-tabs-mode: nil; -*-

// we only include RcppEigen.h which pulls Rcpp.h in for us
#include <RcppEigen.h>
#include <math.h>
#include <iostream>
#include <float.h>


// [[Rcpp::export]]
std::list < std::vector < double> >  llreg2(Eigen::MatrixXd Responses,Eigen::MatrixXd COVARIATES, std::vector<double> Bwknn_ratio ) 
{

	int n, p;

	Rcpp::Function rcond("rcond");
	
	n = COVARIATES.rows();
	p = COVARIATES.cols();

	Eigen::MatrixXd Transposed_COVARIATES = COVARIATES.transpose();

	Eigen::MatrixXd PROX(n,n), PROXSORTED(n,n);

	// Note the memory savings in calculating PROX
	// Exploited the symmetry to make it twice as fast

	double sum,diff;

	for (int ii = 0; ii < n; ii++)
	{

		for (int jj = 0; jj < ii+1; jj++)
		{

			diff = (Transposed_COVARIATES.col(ii) - Transposed_COVARIATES.col(jj)).norm();
			PROX(ii,jj) = diff;
			PROX(jj,ii) = diff;

		}

	}

	PROX.diagonal() = PROX.colwise().maxCoeff();

	for (int ii = 0; ii < n; ii++)
	{
		Eigen::VectorXd v = PROX.col(ii);
		std::sort(v.data(), v.data()+v.size());
		PROXSORTED.col(ii) = v;
	}



	int nbbw = Bwknn_ratio.size();
	std::vector<int> Bwknn(nbbw);
	Eigen::MatrixXd BW(nbbw,n);

	for (int ii = 0; ii < nbbw; ii++)
	{
		Bwknn[ii]  = ceil(Bwknn_ratio[ii]*n)-1;
		BW.row(ii) = PROXSORTED.row(Bwknn[ii]);
	}
	


	// Remove 0s and replace them wit something small. NOTE: We can make it smaller (DBL_MIN?)


	double max_entry = BW.maxCoeff();

	for (size_t ii = 0, size = BW.size(); ii < size; ii++)
	{
		
		if (*(BW.data()+ ii) <= 0)
		{
			*(BW.data()+ ii) = max_entry + 10;
		}

	}

	double min_non_zero_entry = BW.minCoeff()/2;

	for (size_t ii = 0, size = BW.size(); ii < size; ii++)
	{
		
		if (*(BW.data()+ ii) > max_entry + 5)
		{
			*(BW.data()+ ii) = min_non_zero_entry;
		}

	}

	// count the largest number of occurences of the smallest value in each row)


	int max_ties = 0, counts = 0;
	double local_min = 0.0;

	for (int ii = 0; ii < n; ii++)
	{

		Eigen::VectorXd v = BW.col(ii);
		double local_min = v.minCoeff();
		counts = (v.array() <= local_min).count();

		if (max_ties < counts)
		{
			max_ties = counts;
		}

	}

	// Run the leave-one-out cross validation:

	Eigen::MatrixXd ESTIMATION(nbbw - max_ties + 1,n);
	std::vector < double > Loocv(nbbw - max_ties + 1);

	
	Eigen::MatrixXd KERNEL(n,n);
	Eigen::MatrixXd XBAR(n,p);
	std::vector<double> Ybar(n);
	std::vector<double> rowsums(n);
	Eigen::MatrixXd KERNEL_SCALED(n,n);
	Eigen::MatrixXd tmp_mat(n,p), CCOVAR(n,p);
	Eigen::MatrixXd T12(n,p);

	for (int ll = max_ties-1; ll < nbbw; ll ++)
	{

		// Calculate row sums and Kernel
		double tmp, new_entry;

		for (int ii = 0; ii < n; ii++)
		{

			Eigen::VectorXd  Dist = PROX.row(ii)/BW(ll,ii);
			KERNEL.row(ii) = 1 - Dist.array()*Dist.array();
				
		}


		for (size_t ii = 0, size = KERNEL.size(); ii < size; ii++)
		{
		
			if (*(KERNEL.data()+ ii) > 1)
			{
				*(KERNEL.data()+ ii) = 0;
			}

			if (*(KERNEL.data()+ ii) < 0)
			{
				*(KERNEL.data()+ ii) = 0;
			}

		}

		KERNEL.diagonal().setZero();

		for (int ii = 0; ii < n; ii++)
		{
			
			double row_sum = KERNEL.row(ii).sum();

			if (row_sum <= 0)
			{

				// Should we set the diagonal element to 0?????
				KERNEL.row(ii).setOnes();
				row_sum = n;

			}

			rowsums[ii] = row_sum;

		}

		// Regression happes below: 


		Eigen::MatrixXd Ynum = KERNEL * Responses;

		for (int ii = 0; ii < n; ii++)
		{
			Ybar[ii] = Ynum(ii,0)/rowsums[ii];
		}
		
		for (int ii = 0; ii < n; ii++)
		{
			KERNEL_SCALED.row(ii) = KERNEL.col(ii).array()*Responses(ii,0);
		}

		Eigen::MatrixXd NUM = KERNEL * COVARIATES;

		for (int ii = 0; ii < n; ii++)
		{
			XBAR.row(ii) = NUM.row(ii)/rowsums[ii];
		}

		for (int ii = 0; ii < n; ii++)
		{
			T12.row(ii) = XBAR.row(ii)*Ynum(ii,0);
		}

		Eigen::MatrixXd T1  = KERNEL_SCALED.transpose()*COVARIATES - T12;

		static double threshold = DBL_MIN/0.00000000001;
		
		
		for (int ii = 0; ii < n; ii ++)
		{
			

			for (int jj = 0; jj < p; jj++)
			{

				CCOVAR.col(jj) = COVARIATES.col(jj).array() - XBAR(ii,jj);

			}
			
			for (int jj = 0; jj < n; jj++)
			{
			
				tmp_mat.row(jj) = CCOVAR.row(jj).array()*KERNEL(ii,jj);

			}

			Eigen::MatrixXd T2 = CCOVAR.transpose() * tmp_mat;

			// Calling rcond has a moderate overhead. Hence we stick with computing the SVD unless p is greater than 10. 

			if (p < 10){ 

				Eigen::JacobiSVD<Eigen::MatrixXd> svd(T2,Eigen::ComputeThinU | Eigen::ComputeThinV);
				if (svd.singularValues()(svd.singularValues().size()-1) <= 0.00000000001*(svd.singularValues()(0)))
				{
					ESTIMATION(ll - max_ties + 1,ii) = Ybar[ii];
				}
				else
				{
					Eigen::VectorXd Beta = svd.solve(T1.transpose().col(ii));
					ESTIMATION(ll - max_ties + 1,ii) = Ybar[ii] + Beta.dot(CCOVAR.row(ii));
				}
				

			} else {

				double rcond_value = Rcpp::as<double>(rcond(T2));
				if (rcond_value <= 0.00000000001)
				{
					ESTIMATION(ll - max_ties + 1,ii) = Ybar[ii];
				} 
				else
				{
					Eigen::VectorXd Beta = T2.colPivHouseholderQr().solve(T1.transpose().col(ii));
					ESTIMATION(ll - max_ties + 1,ii) = Ybar[ii] + Beta.dot(CCOVAR.row(ii));
				}

			}     

			/*double rcond_value = Rcpp::as<double>(rcond(T2));

			//Eigen::JacobiSVD<Eigen::MatrixXd> svd(T2,Eigen::ComputeThinU | Eigen::ComputeThinV);
			//Eigen::BDCSVD<Eigen::MatrixXd> svd(T2,Eigen::ComputeThinU | Eigen::ComputeThinV);

			if (rcond_value <= 0.00000000001)
			//if (svd.singularValues()(svd.singularValues().size()-1) <= 0.00000000001*(svd.singularValues()(0)))
			{
				ESTIMATION(ll - max_ties + 1,ii) = Ybar[ii];
			} 
			//else if ( (svd.singularValues()(0)) <= threshold)
			//{
			//	ESTIMATION(ll - max_ties + 1 ,ii) = Ybar[ii];
			//}
			else
			{
				//Eigen::VectorXd Beta = svd.solve(T1.transpose().col(ii));
				Eigen::VectorXd Beta = T2.colPivHouseholderQr().solve(T1.transpose().col(ii));
				ESTIMATION(ll - max_ties + 1,ii) = Ybar[ii] + Beta.dot(CCOVAR.row(ii));
			} */

		}

		Loocv[ll - max_ties + 1] = (Responses.transpose().row(0)-ESTIMATION.row(ll - max_ties + 1)).squaredNorm()/n;

	}

	int minElementIndex = std::min_element(Loocv.begin(),Loocv.end()) - Loocv.begin();
	
	std::list < std::vector < double > > Output;
	
	Eigen::VectorXd mat = ESTIMATION.row(minElementIndex); 	
	std::vector<double> vec(mat.data(), mat.data() + mat.size());
	std::vector<double> vec2{Loocv[minElementIndex]};
	std::vector<double> vec3{Bwknn_ratio[minElementIndex]};
	Output.push_back(vec);
	Output.push_back(vec2);
	Output.push_back(vec3);
	                     
	return Output;

}














